@@ -58,7 +58,7 @@ def mro_items(type_obj):
     """Takes a type and returns an iterator over all class variables
     throughout the type hierarchy (respecting the MRO).
 
-    >>> sorted(set([k for k, v in mro_items(int) if not k.startswith('__') and not callable(v)]))
+    >>> sorted(set([k for k, v in mro_items(int) if not k.startswith('__') and 'bytes' not in k and not callable(v)]))
     ['denominator', 'imag', 'numerator', 'real']
     """
     # TODO: handle slots?
@@ -262,30 +262,50 @@ def wraps(func, injected=None, **kw):
         func (function): The callable whose attributes are to be copied.
         injected (list): An optional list of argument names which
             should not appear in the new wrapper's signature.
-        update_dict(bool): Whether to copy other, non-standard
+        update_dict (bool): Whether to copy other, non-standard
             attributes of *func* over to the wrapper. Defaults to True.
+        inject_to_varkw (bool): Ignore missing arguments when a
+            ``**kwargs``-type catch-all is present. Defaults to True.
 
     For more in-depth wrapping of functions, see the
     :class:`FunctionBuilder` type, on which wraps was built.
+
     """
+    # TODO: maybe automatically use normal wraps in the very rare case
+    # that the signatures actually match and no adapter is needed.
     if injected is None:
         injected = []
     elif isinstance(injected, basestring):
         injected = [injected]
+    else:
+        injected = list(injected)
+
+    if isinstance(func, (classmethod, staticmethod)):
+        raise TypeError('wraps does not support wrapping classmethods and'
+                        ' staticmethods, change the order of wrapping to'
+                        ' wrap the underlying function: %r'
+                        % (getattr(func, '__func__', None),))
 
     update_dict = kw.pop('update_dict', True)
+    inject_to_varkw = kw.pop('inject_to_varkw', True)
     if kw:
         raise TypeError('unexpected kwargs: %r' % kw.keys())
 
     fb = FunctionBuilder.from_func(func)
     for arg in injected:
-        fb.remove_arg(arg)
+        try:
+            fb.remove_arg(arg)
+        except MissingArgument:
+            if inject_to_varkw and fb.varkw is not None:
+                continue  # keyword arg will be caught by the varkw
+            raise
 
     fb.body = 'return _call(%s)' % fb.get_invocation_str()
 
     def wrapper_wrapper(wrapper_func):
         execdict = dict(_call=wrapper_func, _func=func)
         fully_wrapped = fb.get_func(execdict, with_dict=update_dict)
+        fully_wrapped.__wrapped__ = func  # ref to the original function (#115)
 
         return fully_wrapped
 
@@ -341,6 +361,8 @@ class FunctionBuilder(object):
             but only for the *kwonlyargs*. **Python 3 only.**
         annotations (dict): Mapping of type hints and so
             forth. **Python 3 only.**
+        filename (str): The filename that will appear in
+            tracebacks. Defaults to "boltons.funcutils.FunctionBuilder".
         indent (int): Number of spaces with which to indent the
             function *body*. Values less than 1 will result in an error.
         dict (dict): Any other attributes which should be added to the
@@ -386,7 +408,8 @@ class FunctionBuilder(object):
                  'dict': dict,
                  'module': lambda: None,
                  'body': lambda: 'pass',
-                 'indent': lambda: 4}
+                 'indent': lambda: 4,
+                 'filename': lambda: 'boltons.funcutils.FunctionBuilder'}
 
     _defaults.update(_argspec_defaults)
 
@@ -458,6 +481,9 @@ class FunctionBuilder(object):
         """
         # TODO: copy_body? gonna need a good signature regex.
         # TODO: might worry about __closure__?
+        if not callable(func):
+            raise TypeError('expected callable object, not %r' % (func,))
+
         kwargs = {'name': func.__name__,
                   'doc': func.__doc__,
                   'module': func.__module__,
@@ -539,15 +565,18 @@ class FunctionBuilder(object):
         try:
             self.args.remove(arg_name)
         except ValueError:
-            raise ValueError('arg %r not found in %s argument list: %r'
-                             % (arg_name, self.name, self.args))
+            exc = MissingArgument('arg %r not found in %s argument list: %r'
+                                  % (arg_name, self.name, self.args))
+            exc.arg_name = arg_name
+            raise exc
         d_dict.pop(arg_name, None)
         self.defaults = tuple([d_dict[a] for a in self.args if a in d_dict])
         return
 
     def _compile(self, src, execdict):
-        filename = ('<boltons.funcutils.FunctionBuilder-%d>'
-                    % (next(self._compile_count),))
+
+        filename = ('<%s-%d>'
+                    % (self.filename, next(self._compile_count),))
         try:
             code = compile(src, filename, 'single')
             exec(code, execdict)
@@ -556,6 +585,10 @@ class FunctionBuilder(object):
         return execdict
 
 
+class MissingArgument(ValueError):
+    pass
+
+
 def _indent(text, margin, newline='\n', key=bool):
     "based on boltons.strutils.indent"
     indented_lines = [(margin + line if key(line) else line)
@@ -563,4 +596,57 @@ def _indent(text, margin, newline='\n', key=bool):
     return newline.join(indented_lines)
 
 
+try:
+    from functools import total_ordering  # 2.7+
+except ImportError:
+    # python 2.6
+    def total_ordering(cls):
+        """Class decorator that fills in missing comparators/ordering
+        methods. Backport of :func:`functools.total_ordering` to work
+        with Python 2.6.
+
+        Code from http://code.activestate.com/recipes/576685/
+        """
+        convert = {
+            '__lt__': [
+                ('__gt__',
+                 lambda self, other: not (self < other or self == other)),
+                ('__le__',
+                 lambda self, other: self < other or self == other),
+                ('__ge__',
+                 lambda self, other: not self < other)],
+            '__le__': [
+                ('__ge__',
+                 lambda self, other: not self <= other or self == other),
+                ('__lt__',
+                 lambda self, other: self <= other and not self == other),
+                ('__gt__',
+                 lambda self, other: not self <= other)],
+            '__gt__': [
+                ('__lt__',
+                 lambda self, other: not (self > other or self == other)),
+                ('__ge__',
+                 lambda self, other: self > other or self == other),
+                ('__le__',
+                 lambda self, other: not self > other)],
+            '__ge__': [
+                ('__le__',
+                 lambda self, other: (not self >= other) or self == other),
+                ('__gt__',
+                 lambda self, other: self >= other and not self == other),
+                ('__lt__',
+                 lambda self, other: not self >= other)]
+        }
+        roots = set(dir(cls)) & set(convert)
+        if not roots:
+            raise ValueError('must define at least one ordering operation:'
+                             ' < > <= >=')
+        root = max(roots)       # prefer __lt__ to __le__ to __gt__ to __ge__
+        for opname, opfunc in convert[root]:
+            if opname not in roots:
+                opfunc.__name__ = opname
+                opfunc.__doc__ = getattr(int, opname).__doc__
+                setattr(cls, opname, opfunc)
+        return cls
+
 # end funcutils.py
